/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "thread.h"

void threadInit(thread_t *thread, threadfunction_t *call) {
  ASSERT_NOT_NULL(thread);
  ASSERT_NOT_NULL(call);

  thread->call = call;
  thread->user = NULL;
  thread->state = 0x00;
}

void threadStart(thread_t *thread) {
  ASSERT_NOT_NULL(thread);
  ASSERT_FLAG_OFF(thread->state, THREAD_FLAG_RUNNING);
  ASSERT_FLAG_OFF(thread->state, THREAD_FLAG_QUEUED);

  thread->state |= THREAD_FLAG_QUEUED;
  sysThreadCreate(&_threadWrappedCallback, thread->pt, thread);
}

void threadJoin(thread_t *thread) {
  ASSERT_NOT_NULL(thread);
  ASSERT_FLAG_ON(thread->state, THREAD_FLAG_RUNNING);

  sysThreadJoin(thread->pt);
}

void threadCancel(thread_t *thread) {
  ASSERT_NOT_NULL(thread);
  ASSERT_FLAG_ON(thread->state, THREAD_FLAG_RUNNING);

  sysThreadCancel(thread->pt, 1);
}

void threadSleep(float time) {
  ASSERT_GREATER_THAN(time, 0);

  sleep((unsigned long)(time * 1000.0f));
}

int32_t _threadWrappedCallback(thread_t *thread) {
  int32_t response;
  
  ASSERT_NOT_NULL(thread);
  
  thread->state |= THREAD_FLAG_RUNNING;
  thread->state = flagOff(thread->state, THREAD_FLAG_QUEUED);

  response = thread->call(thread);

  thread->state |= THREAD_FLAG_HAS_RUN;
  thread->state = flagOff(thread->state, THREAD_FLAG_RUNNING);
  
  return response;
}