/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../libs.h"
#include "../assert/assert.h"
#include "../util/flags.h"

typedef struct _thread_t thread_t;

#define THREAD_FLAG_RUNNING flagDefine(0)
#define THREAD_FLAG_QUEUED flagDefine(1)
#define THREAD_FLAG_HAS_RUN flagDefine(2)

/**
 * Thread function itself.
 * 
 * @param thread Thread that was called.
 * @return The exit code, try to ensure only 0 is possible.
 */
typedef int32_t threadfunction_t(thread_t *thread);

typedef struct _thread_t {
  threadhandle_t pt;
  void *user;
  uint8_t state;
  threadfunction_t *call;
} thread_t;

/**
 * Initialize a thread, prepare it to be started. You need to manually start it
 * yourself.
 * 
 * @param thread Thread pointer to maintain.
 * @param call User callback to fire when the thread is started.
 */
void threadInit(thread_t *thread, threadfunction_t *call);

/**
 * Start a previously prepared thread.
 * 
 * @param thread Thread to start.
 */
void threadStart(thread_t *thread);

/**
 * Wait for the given thread to finish execution.
 * 
 * @param thread Thread to join.
 */
void threadJoin(thread_t *thread);

/**
 * Cancel a running thread, avoid using where possible incase of any undefined
 * behaviour.
 * 
 * @param thread Thread to cancel.
 */
void threadCancel(thread_t *thread);

/**
 * Sleep the current thread.
 * 
 * @param time Time in seconds to sleep for.
 */
void threadSleep(float time);

/** Managing function for thread to help update the state */
int32_t _threadWrappedCallback(thread_t *thread);